@@ -4,24 +4,39 @@ import serial
 from time import sleep
 
 
-NUM_KEY_DIGITS = 16
-COMMANDS = {'+++':       'Enter AT mode',
-            'ATI':       'Shows the radio version',
-            'ATI2':      'Shows the board type',
-            'ATI3':      'Shows board frequency',
-            'ATI4':      'Shows board version',
-            'ATI5':      'Shows all user settable EEPROM parameters',
-            'ATI6':      'Displays TDM timing report',
-            'ATI7':      'Displays RSSI signal report',
-            'ATO':       'Exits AT command mode',
-            'AT{n}?':    'Displays radio parameter number ‘n’',
-            'AT{n}={X}': 'Sets radio parameter number ‘n’ to ‘X’',
-            'ATZ':       'Reboots the radio',
-            'AT&W':      'Writes current parameters to EEPROM',
-            'AT&F':      'Resets all parameters to factory defaults',
-            'AT&T=RSSI': 'Enables RSSI debugging report',
-            'AT&T=TDM':  'Enables TDM debugging report',
-            'AT&T':      'Disables debugging report'}
+NUM_KEY_DIGITS = 32
+SERIAL_SPEEDS  = [1, 2, 4, 9, 38, 57, 115, 230, 460, 1000]
+AIR_SPEEDS     = [12, 56, 64, 100, 125, 200, 224, 500, 750]
+ANT_MODES      = [1, 2, 3]
+NET_IDS        = list(range(500))
+TX_PWRS        = list(range(31))
+MIN_FREQS      = list(range(895000, 936000, 1000))
+MAX_FREQS      = list(range(895000, 936000, 1000))
+NUM_CHANNELS   = list(range(1, 51))
+DUTY_CYCLES    = list(range(10, 110, 10))
+LBT_RSSIS      = [0, 25, 50, 100, 150, 200, 220]
+MAX_WINDOWS    = list(range(33, 132))
+COMMANDS       = {'+++':       'Enter AT mode',
+                  'ATI':       'Shows the radio version',
+                  'ATI2':      'Shows the board type',
+                  'ATI3':      'Shows board frequency',
+                  'ATI4':      'Shows board version',
+                  'ATI5':      'Shows all user settable EEPROM parameters and their values',
+                  'ATI5?':     'Shows all user settable EEPROM parameters and their possible range',
+                  'ATI6':      'Displays TDM timing report',
+                  'ATI7':      'Displays RSSI signal report',
+                  'ATI8':      'Display Device 64 bit unique ID',
+                  'ATI9':      'Display node ID [multipoint only]',
+                  'ATO':       'Exits AT command mode',
+                  'AT{n}?':    'Displays radio parameter number ‘n’',
+                  'AT{n}={X}': 'Sets radio parameter number ‘n’ to ‘X’',
+                  'ATZ':       'Reboots the radio',
+                  'AT&W':      'Writes current parameters to EEPROM',
+                  'AT&F':      'Resets all parameters to factory defaults',
+                  'AT&T=RSSI': 'Enables RSSI debugging report',
+                  'AT&T=TDM':  'Enables TDM debugging report',
+                  'AT&UPDATE': 'Reset and enter boot mode',
+                  'AT&T':      'Disables debugging report'}
 
 
 def gen_key(num_digits=NUM_KEY_DIGITS):
@@ -39,6 +54,8 @@ class RFDConfig(object):
                                            'maxVal':          None,
                                            'curVal':          None,
                                            'desVal':          None,
+                                           'curValRemote':    None,
+                                           'desValRemote':    None,
                                            'sameOnAllModems': False},
                        'boardType':       {'id':              'I2',
                                            'description':     'Board type',
@@ -47,6 +64,8 @@ class RFDConfig(object):
                                            'maxVal':          None,
                                            'curVal':          None,
                                            'desVal':          None,
+                                           'curValRemote':    None,
+                                           'desValRemote':    None,
                                            'sameOnAllModems': False},
                        'boardFreq':       {'id':              'I3',
                                            'description':     'Board frequency',
@@ -55,6 +74,8 @@ class RFDConfig(object):
                                            'maxVal':          None,
                                            'curVal':          None,
                                            'desVal':          None,
+                                           'curValRemote':    None,
+                                           'desValRemote':    None,
                                            'sameOnAllModems': False},
                        'boardVersion':    {'id':              'I4',
                                            'description':     'Board version',
@@ -63,6 +84,8 @@ class RFDConfig(object):
                                            'maxVal':          None,
                                            'curVal':          None,
                                            'desVal':          None,
+                                           'curValRemote':    None,
+                                           'desValRemote':    None,
                                            'sameOnAllModems': False},
                        'FORMAT':          {'id':              'S0',
                                            'description':     'This is for EEPROM version, can’t be changed',
@@ -71,6 +94,8 @@ class RFDConfig(object):
                                            'maxVal':          None,
                                            'curVal':          None,
                                            'desVal':          None,
+                                           'curValRemote':    None,
+                                           'desValRemote':    None,
                                            'sameOnAllModems': False},
                        'SERIAL_SPEED':    {'id':              'S1',
                                            'description':     'Serial speed in ‘one byte form’',
@@ -79,6 +104,8 @@ class RFDConfig(object):
                                            'maxVal':          115,
                                            'curVal':          None,
                                            'desVal':          None,
+                                           'curValRemote':    None,
+                                           'desValRemote':    None,
                                            'sameOnAllModems': False},
                        'AIR_SPEED':       {'id':              'S2',
                                            'description':     'Air data rate in one byte form',
@@ -87,6 +114,8 @@ class RFDConfig(object):
                                            'maxVal':          250,
                                            'curVal':          None,
                                            'desVal':          None,
+                                           'curValRemote':    None,
+                                           'desValRemote':    None,
                                            'sameOnAllModems': True},
                        'NETID':           {'id':              'S3',
                                            'description':     'Network ID. It should be the same on both modems',
@@ -95,6 +124,8 @@ class RFDConfig(object):
                                            'maxVal':          499,
                                            'curVal':          None,
                                            'desVal':          None,
+                                           'curValRemote':    None,
+                                           'desValRemote':    None,
                                            'sameOnAllModems': True},
                        'TXPOWER':         {'id':              'S4',
                                            'description':     'Transmit power in dBm. Maximum is 30dBm',
@@ -103,6 +134,8 @@ class RFDConfig(object):
                                            'maxVal':          30,
                                            'curVal':          None,
                                            'desVal':          None,
+                                           'curValRemote':    None,
+                                           'desValRemote':    None,
                                            'sameOnAllModems': False},
                        'ECC':             {'id':              'S5',
                                            'description':     'Enables or disables the golay error correcting code',
@@ -111,6 +144,8 @@ class RFDConfig(object):
                                            'maxVal':          1,
                                            'curVal':          None,
                                            'desVal':          None,
+                                           'curValRemote':    None,
+                                           'desValRemote':    None,
                                            'sameOnAllModems': True},
                        'MAVLINK':         {'id':              'S6',
                                            'description':     'Enables or disables the MAVLink framing and reporting',
@@ -119,6 +154,8 @@ class RFDConfig(object):
                                            'maxVal':          1,
                                            'curVal':          None,
                                            'desVal':          None,
+                                           'curValRemote':    None,
+                                           'desValRemote':    None,
                                            'sameOnAllModems': False},
                        'OP_RESEND':       {'id':              'S7',
                                            'description':     'Opportunic Resend',
@@ -127,6 +164,8 @@ class RFDConfig(object):
                                            'maxVal':          1,
                                            'curVal':          None,
                                            'desVal':          None,
+                                           'curValRemote':    None,
+                                           'desValRemote':    None,
                                            'sameOnAllModems': False},
                        'MIN_FREQ':        {'id':              'S8',
                                            'description':     'Min freq in KHz',
@@ -135,6 +174,8 @@ class RFDConfig(object):
                                            'maxVal':          927000,
                                            'curVal':          None,
                                            'desVal':          None,
+                                           'curValRemote':    None,
+                                           'desValRemote':    None,
                                            'sameOnAllModems': True},
                        'MAX_FREQ':        {'id':              'S9',
                                            'description':     'Max freq in KHz',
@@ -143,6 +184,8 @@ class RFDConfig(object):
                                            'maxVal':          928000,
                                            'curVal':          None,
                                            'desVal':          None,
+                                           'curValRemote':    None,
+                                           'desValRemote':    None,
                                            'sameOnAllModems': True},
                        'NUM_CHANNELS':    {'id':              'S10',
                                            'description':     'Number of frequency hopping channels',
@@ -151,6 +194,8 @@ class RFDConfig(object):
                                            'maxVal':          50,
                                            'curVal':          None,
                                            'desVal':          None,
+                                           'curValRemote':    None,
+                                           'desValRemote':    None,
                                            'sameOnAllModems': True},
                        'DUTY_CYCLE':      {'id':              'S11',
                                            'description':     'The percentage of time to allow transmit',
@@ -159,6 +204,8 @@ class RFDConfig(object):
                                            'maxVal':          100,
                                            'curVal':          None,
                                            'desVal':          None,
+                                           'curValRemote':    None,
+                                           'desValRemote':    None,
                                            'sameOnAllModems': False},
                        'LBT_RSSI':        {'id':              'S13',
                                            'description':     'Listen before talk threshold (This parameter shouldn’t be changed)',
@@ -167,6 +214,8 @@ class RFDConfig(object):
                                            'maxVal':          1,
                                            'curVal':          None,
                                            'desVal':          None,
+                                           'curValRemote':    None,
+                                           'desValRemote':    None,
                                            'sameOnAllModems': True},
                        'RTSCTS':          {'id':              'S13',
                                            'description':     'Ready-to-send and Clear-to-send flow control',
@@ -175,6 +224,8 @@ class RFDConfig(object):
                                            'maxVal':          1,
                                            'curVal':          None,
                                            'desVal':          None,
+                                           'curValRemote':    None,
+                                           'desValRemote':    None,
                                            'sameOnAllModems': False},
                        'MAX_WINDOW':      {'id':              'S14',
                                            'description':     'Max transit window size used to limit max time/latency if required otherwise will be set automatically',
@@ -183,6 +234,8 @@ class RFDConfig(object):
                                            'maxVal':          400,
                                            'curVal':          None,
                                            'desVal':          None,
+                                           'curValRemote':    None,
+                                           'desValRemote':    None,
                                            'sameOnAllModems': False},
                        'ENCRYPTION_LEVEL':{'id':              'S15',
                                            'description':     'Encryption level 0=off, 1=128bit AES',
@@ -191,6 +244,8 @@ class RFDConfig(object):
                                            'maxVal':          1,
                                            'curVal':          None,
                                            'desVal':          None,
+                                           'curValRemote':    None,
+                                           'desValRemote':    None,
                                            'sameOnAllModems': True},
                        'GPI1_1R/CIN':     {'id':              'S16',
                                            'description':     'Set GPIO 1.1 (pin 15) as R/C(PPM) input',
@@ -199,6 +254,8 @@ class RFDConfig(object):
                                            'maxVal':          1,
                                            'curVal':          None,
                                            'desVal':          None,
+                                           'curValRemote':    None,
+                                           'desValRemote':    None,
                                            'sameOnAllModems': False},
                        'GPO1_1R/COUT':    {'id':              'S17',
                                            'description':     'Set GPIO 1.1 (pin 15) as R/C(PPM) output',
@@ -207,6 +264,8 @@ class RFDConfig(object):
                                            'maxVal':          1,
                                            'curVal':          None,
                                            'desVal':          None,
+                                           'curValRemote':    None,
+                                           'desValRemote':    None,
                                            'sameOnAllModems': False},
                        'GPO1_1SBUSIN':    {'id':              'S18',
                                            'description':     'Set GPIO 1.1 (pin 12) as SBUS input',
@@ -215,6 +274,8 @@ class RFDConfig(object):
                                            'maxVal':          1,
                                            'curVal':          None,
                                            'desVal':          None,
+                                           'curValRemote':    None,
+                                           'desValRemote':    None,
                                            'sameOnAllModems': False},
                        'GPO1_1SBUSOUT':    {'id':             'S19',
                                            'description':     'Set GPIO 1.1 (pin 12) as SBUS output',
@@ -223,6 +284,8 @@ class RFDConfig(object):
                                            'maxVal':          4,
                                            'curVal':          None,
                                            'desVal':          None,
+                                           'curValRemote':    None,
+                                           'desValRemote':    None,
                                            'sameOnAllModems': False},
                        'ANT_MODE':        {'id':              'S20',
                                            'description':     '0= Diversity, 1= Antenna 1 only, 2= Antenna 2 only, 3= Antenna 1 TX and antenna 2 RX',
@@ -231,6 +294,8 @@ class RFDConfig(object):
                                            'maxVal':          3,
                                            'curVal':          None,
                                            'desVal':          None,
+                                           'curValRemote':    None,
+                                           'desValRemote':    None,
                                            'sameOnAllModems': False},
                        'GPO1_3STATLED ':  {'id':              'S21',
                                            'description':     'Set GPIO 1.1 (pin 12) as output with state that mirrors the status LED on the modem',
@@ -239,6 +304,8 @@ class RFDConfig(object):
                                            'maxVal':          1,
                                            'curVal':          None,
                                            'desVal':          None,
+                                           'curValRemote':    None,
+                                           'desValRemote':    None,
                                            'sameOnAllModems': False},
                        'GPO1_0TXEN485':   {'id':              'S22',
                                            'description':     'Set GPIO 1.0 (pin 13) as control signal on DINIO and RS485 interface boards.',
@@ -247,6 +314,8 @@ class RFDConfig(object):
                                            'maxVal':          1,
                                            'curVal':          None,
                                            'desVal':          None,
+                                           'curValRemote':    None,
+                                           'desValRemote':    None,
                                            'sameOnAllModems': False},
                        'RATE/FREQBAND':   {'id':              'S23',
                                            'description':     'Changes the frequencies bands and airspeeds within set ranges on compliant modems ensuring compliance is maintained',
@@ -255,6 +324,8 @@ class RFDConfig(object):
                                            'maxVal':          3,
                                            'curVal':          None,
                                            'desVal':          None,
+                                           'curValRemote':    None,
+                                           'desValRemote':    None,
                                            'sameOnAllModems': True},
                        'TARGET_RSSI':     {'id':              'R0',
                                            'description':     'Optimal RSSI value to try to sustain (off = 255)',
@@ -263,6 +334,8 @@ class RFDConfig(object):
                                            'maxVal':          255,
                                            'curVal':          None,
                                            'desVal':          None,
+                                           'curValRemote':    None,
+                                           'desValRemote':    None,
                                            'sameOnAllModems': False},
                        'HYSTERESIS_RSSI': {'id':              'R1',
                                            'description':     'Amount of change before power levels altered',
@@ -271,14 +344,18 @@ class RFDConfig(object):
                                            'maxVal':          50,
                                            'curVal':          None,
                                            'desVal':          None,
+                                           'curValRemote':    None,
+                                           'desValRemote':    None,
                                            'sameOnAllModems': False},
-                       'Encryption':      {'id':              'E',
+                       'EncryptionKey':   {'id':              'E',
                                            'description':     'AES encryption key',
                                            'defaultVal':      None,
                                            'minVal':          None,
                                            'maxVal':          None,
                                            'curVal':          None,
                                            'desVal':          None,
+                                           'curValRemote':    None,
+                                           'desValRemote':    None,
                                            'sameOnAllModems': None},
                        'Print':           {'id':              'PP',
                                            'description':     'Print all Pin Settings',
@@ -287,6 +364,8 @@ class RFDConfig(object):
                                            'maxVal':          None,
                                            'curVal':          None,
                                            'desVal':          None,
+                                           'curValRemote':    None,
+                                           'desValRemote':    None,
                                            'sameOnAllModems': None},
                        'Input':           {'id':              'PI={x}',
                                            'description':     'Set Pin x to Input',
@@ -295,6 +374,8 @@ class RFDConfig(object):
                                            'maxVal':          None,
                                            'curVal':          None,
                                            'desVal':          None,
+                                           'curValRemote':    None,
+                                           'desValRemote':    None,
                                            'sameOnAllModems': None},
                        'Read':            {'id':              'PR={x}',
                                            'description':     'Read Pin X value (When set to input)',
@@ -303,6 +384,8 @@ class RFDConfig(object):
                                            'maxVal':          None,
                                            'curVal':          None,
                                            'desVal':          None,
+                                           'curValRemote':    None,
+                                           'desValRemote':    None,
                                            'sameOnAllModems': None},
                        'Output':          {'id':              'PO={x}',
                                            'description':     'Set Pin x to Output (Default) can only be controlled by ATPC',
@@ -311,12 +394,16 @@ class RFDConfig(object):
                                            'maxVal':          None,
                                            'curVal':          None,
                                            'desVal':          None,
+                                           'curValRemote':    None,
+                                           'desValRemote':    None,
                                            'sameOnAllModems': None},
                        'ControlOn':       {'id':              'PC={x},1',
                                            'description':     'Turn pin x on -­‐ Output Mode / Set internal pull up resistor -­‐ Input Mode',
                                            'defaultVal':      None,
                                            'minVal':          None,
                                            'maxVal':          None,
+                                           'curValRemote':    None,
+                                           'desValRemote':    None,
                                            'sameOnAllModems': None},
                        'ControlOff':      {'id':              'PC={x},0',
                                            'description':     'Turn pin x off -­‐ Output Mode / Set internal pull down resistor -­‐ Input Mode',
@@ -325,6 +412,8 @@ class RFDConfig(object):
                                            'maxVal':          None,
                                            'curVal':          None,
                                            'desVal':          None,
+                                           'curValRemote':    None,
+                                           'desValRemote':    None,
                                            'sameOnAllModems': None}}
     
     def send(self, command, timeout=0.01):
@@ -446,30 +535,48 @@ class RFDConfig(object):
             self.send('ATO')
             self.port.close()
     
-    def loadParam(self, param):
+    def loadParam(self, param, local=True):
         if self.port.isOpen():
             if param in self.params.keys():
                 if '=' in self.params[param]['id']:
-                    return
+                    return False
                 
                 if 'I' not in self.params[param]['id']:
-                    response = self.send_and_rec('AT{}?'.format(self.params[param]['id']))
+                    if local:
+                        response = self.send_and_rec('AT{}?'.format(self.params[param]['id']))
+                    else:
+                        response = self.send_and_rec('RT{}?'.format(self.params[param]['id']))
                 else:
-                    response = self.send_and_rec('AT{}'.format(self.params[param]['id']))
+                    if local:
+                        response = self.send_and_rec('AT{}'.format(self.params[param]['id']))
+                    else:
+                        response = self.send_and_rec('RT{}'.format(self.params[param]['id']))
                 
                 if len(response.split('\r\n')) >= 2:
                     response = response.split('\r\n')[1:-1]
                     response = '\n'.join(response)
                     
                     if response.upper() == 'ERROR':
-                        self.params[param]['curVal'] = None
+                        if local:
+                            self.params[param]['curVal'] = None
+                        else:
+                            self.params[param]['curValRemote'] = None
                     else:
                         try:
-                            self.params[param]['curVal'] = int(response)
+                            if local:
+                                self.params[param]['curVal'] = int(response)
+                            else:
+                                self.params[param]['curValRemote'] = int(response)
                         except ValueError:
-                            self.params[param]['curVal'] = response
+                            if local:
+                                self.params[param]['curVal'] = response
+                            else:
+                                self.params[param]['curValRemote'] = response
+                    return True
+                else:
+                    return False
     
-    def loadAll(self):
+    def loadAll(self, local=True):
         '''
         TODO
         '''
@@ -478,7 +585,7 @@ class RFDConfig(object):
             for param in self.params.keys():
                 self.loadParam(param)
     
-    def writeOutParam(self, param):
+    def writeOutParam(self, param, local=True):
         '''
         TODO
         '''
@@ -486,11 +593,14 @@ class RFDConfig(object):
         if self.port.isOpen():
             if param in self.params.keys():
                 if ('S' in self.params[param]['id']) and self.params[param]['desVal']:
-                    response = self.send_and_rec('AT{n}={X}'.format(n=self.params[param]['id'], X=self.params[param]['desVal']))
+                    if local:
+                        response = self.send_and_rec('AT{n}={X}'.format(n=self.params[param]['id'], X=self.params[param]['desVal']))
+                    else:
+                        response = self.send_and_rec('RT{n}={X}'.format(n=self.params[param]['id'], X=self.params[param]['desValRemote']))
                     return self.responseGood(response)
         return False
 
-    def writeOutAll(self):
+    def writeOutAll(self, local=True):
         '''
         TODO
         '''
@@ -498,9 +608,16 @@ class RFDConfig(object):
         if self.port.isOpen():
             for param in self.params.keys():
                 if 'S' in self.params[param]['id']:
-                    self.writeOutParam(param)
+                    self.writeOutParam(param, local)
                     sleep(0.1)
     
+    def hasRemote(self):
+        '''
+        TODO
+        '''
+        
+        return self.responseGood(self.send_and_rec('RTI5?'))
+    
     def enableRSSI(self):
         '''
         TODO
@@ -552,4 +669,16 @@ class RFDConfig(object):
             self.reset()
             self.send('ATO')
             sleep(1)
-            self.open(self.port.port, self.port.baudrate)
\ No newline at end of file
+            self.open(self.port.port, self.port.baudrate)
+
+
+if __name__ == '__main__':
+    import pprint
+    
+    rfd = RFDConfig()
+    rfd.open('COM9', 115200)
+    
+    rfd.loadAll()
+    pprint.pprint(rfd.params)
+    
+    rfd.close()
